;;; hamster-functions.el -- Funktionen fr Dateien des hamsters
;; Copyright (C) 2002 Karl Pflsterer
;; Author: Karl Pflsterer
;; Created: Januar 2002
;; Version: 0.6
;; Keywords: mswin
;;; Commentary:

;; In dieser Datei sind einige, hoffentlich ntzliche Funktionen
;; gesammelt, zur Erleichterung der Erstellung und Bearbeitung von
;; Skriptdateiein und Scoredateien fr den Hamster.
;; Die folgenden Variablen knnen je nach System angepat werden
;; Angenommen wird, das /cygdrive/c als /c gemounted ist.
;; Auf korrekte Gro-, Kleinschreibung der Dateinamen achten.
;; Default fr die Pfade und Dateinamen ist:
;; `ham-fun-root-directory' C:\Programme\Hamster\
;; `ham-fun-news-scorefile' scores.hst
;; `ham-fun-mail-filterfile' mailFilt.hst

;; Die Funktionen werden durch ein `(require hamster-functions)'
;; in der Startdatei vom Gnus geladen. Folgende Funktionen
;; `(define-key gnus-summary-mode-map 'f12 'ham-fun-kill-mid)'
;; `(define-key gnus-summary-mode-map 'f11 'ham-fun-newsscore-subject)'
;; `(define-key gnus-summary-mode-map 'f10 'ham-fun-newsscore-from)'
;; `(define-key gnus-summary-mode-map 'f9  'ham-fun-scorefile-edit)'
;; binden die Funktionen im Summarybuffer an `f9' bis `f12'.
;; Alle Funktionen haben einen docstring, der die Funktion erlutert.

;;; Code:

(eval-when-compile
  (require 'gnus)
  (require 'gnus-sum))

;;; Variablen

(defconst ham-fun-root-directory "/c/Programme/Hamster/"
  "Stammverzeichnis, in dem die Hamsterdateien liegen")

(defconst ham-fun-news-scorefile "scores.hst"
  "Name des News-Scorefiles.")

(defconst ham-fun-mail-filterfile "mailFilt.hst"
  "Name der Mail-Filterdatei.")

(defconst ham-fun-mid-killfile "deletemid.kill"
  "Name eines kill-files. Wird durch ein hamster-Skript ausgelesen.
Dieses Skript nutzt die M-IDs in der Datei, um selektiv Postings aus dem
Hamster-Datenbestand zu lschen.")

(defconst ham-fun-news-subject-lit-prefix " subject \""
  "String im News-Scorefile, der Subject-Filtereintrag kennzeichnet.
->literaler String.")

(defconst ham-fun-news-subject-regexp-prefix " subject {"
  "String im News-Scorefile, der Subject-Filtereintrag kennzeichnet.
->regexp String.")

(defconst ham-fun-news-from-lit-prefix " from \""
  "String im News-Scorefile, der From-Filtereintrag kennzeichnet.
->literaler String.")

(defconst ham-fun-news-from-regexp-prefix " from {"
  "String im news-Scorefile, der From-Filtereintrag kennzeichnet.
->regexp String.")

(defconst ham-fun-score-lit-suffix "\""
  "String im Scorefile, der Filtereintrag abschliet (literaler String).")

(defconst ham-fun-score-regexp-suffix "}"
  "String im Scorefile, der Filtereintrag abschliet (regexp String).")

(defconst ham-fun-default-score "-10000"
  "Default Scorewert.")

(defvar ham-fun-default-abs "[*]"
  "Default Abschnitt. Der Wert ist ein String. Um statt dessen immer den 
aktuellen Newsgruppen-Namen als Default-Abschnitt zu haben, kann man eine
Funktion in den `summary-mode-hook' schreiben, die die Variable mit diesem
Wert belegt.

 (defun ham-fun-set-default-abs ()
   (setq ham-fun-default-abs (ham-fun-newsgroup-name)))
 (add-hook 'summary-mode-hook 'ham-fun-set-deafult-abs)")

(defconst ham-fun-expire-string " Expire:"
  "Leitet den Expire-Zusatz ein.")

(defconst ham-fun-default-expdelta "30"
  "Default-Wert in Tagen, der zum aktuellen Datum addiert wird, um den 
Expire-Eintrag zu ttigen.")

(defconst ham-fun-autostart-string "## Start Automatische Scores"
  "Kennzeichnet Beginn automatischer Eintrge.")

(defconst ham-fun-autoend-string "## Ende Automatische Scores"
  "Kennzeichnet Ende automatischer Eintrge.")



;;; Funktionen

(defun ham-fun-time-to-seconds (time) ;;aus time-date.el
  "Convert time value TIME to a floating point number."
  (+ (* (car time) 65536.0)
     (cadr time)
     (/ (or (nth 2 time) 0) 1000000.0)))

(defun ham-fun-seconds-to-time (seconds)
  "Convert SECONDS (a floating point number) to a time value."
  (list (floor seconds 65536)
	(floor (mod seconds 65536))
	(floor (* (- seconds (ffloor seconds)) 1000000))))

(defun ham-fun-expire-date (&optional exp)
  "Liefert das Expire-Datum im Format YYYYMMDD; zum aktuellen Datum
wird der Wert `ham-fun-default-expdelta' addiert"
  (let* ((deltasec (* (string-to-int(or exp ham-fun-default-expdelta)) 60 60 24))
	 (expdatesec (+ deltasec (ham-fun-time-to-seconds (current-time))))
	 (expdatetime (ham-fun-seconds-to-time expdatesec))
	 (expstr (format-time-string "%Y%m%d" expdatetime)))
    (concat ham-fun-expire-string expstr)))

(defun ham-fun-newsgroup-name ()
 "Gibt Namen der aktuellen Newsgroup in '[]' zurck."
  (concat "[\"" gnus-newsgroup-name "\"]"))

(defmacro ham-fun-article-subject ()    ;von gnus.sum geklaut :-)
  "Return current subject string or nil if nothing."
  `(let ((headers
	  , '(gnus-data-header (assq (gnus-summary-article-number)
				     gnus-newsgroup-data))))
     (and headers
	  (vectorp headers)
	  (mail-header-subject headers))))

(defmacro ham-fun-article-from ()
  "Return current from string or nil if nothing."
  `(let ((headers
	  , '(gnus-data-header (assq (gnus-summary-article-number)
				     gnus-newsgroup-data))))
     (and headers
	  (vectorp headers)
	  (mail-header-from headers))))

(defmacro ham-fun-article-mid ()
  "Return current M-ID string or nil if nothing."
  `(let ((headers
	  , '(gnus-data-header (assq (gnus-summary-article-number)
				     gnus-newsgroup-data))))
     (and headers
	  (vectorp headers)
	  (mail-header-message-id headers))))

(defun ham-fun-scorefile-open (rootdir scorefile)
  "ffnet das Hamsterscorefile.
Als Pfad wird `rootdir' und `scorefile' als Dateiname verwendet.
Defaultwerte sind `ham-fun-root-directory' und `ham-fun-news-scorefile.'"
  (find-file-noselect (concat rootdir scorefile) nil t)
  (set-buffer scorefile))

(defun ham-fun-scorestring-insert (str &optional bool absn)
  "Schreibt den scorestring, sichert Datei und schliet diese.
Wenn noch keine Eintrag in der Datei vorliegt, der automatische Eintrge
kennzeichnet, wird dieser gettigt.

Wenn der optionale Parameter `bool' true ist, wird der
Charakter `\"' gesucht, ein Linebreak eingefuegt, und der String am
aktuellen Point geschrieben.

Wenn nicht, wird nach der Zeichenkette `absn' gesucht. Deren Default ist
\"[*]\". Danach wird der naechste Abschnitt gesucht, um so den zu
bearbeitenden Bereich auf denaktuellen Abschnitt eingrenzen zu
koennen. Um zu erkennen, ob ein bekannter Abschnitt vorgegeben wurde,
wird der Wert von `anf' mit `oldpoint' verglichen. Falls der Abschnitt
vorhanden ist, wird in diesem nach `ham-fun-autostart-string' gesucht
und dahinter der Scoreeintrag geschrieben. Ist
`ham-fun-autostart-string' nicht vorhanden, werden dieser string, der
Scoreeintrag und `ham-fun-autoend-string' geschrieben."
  (if (eq bool t)
      (progn
	(end-of-line)
	(insert "\n" str)
	(save-buffer)
	(kill-buffer nil))
    (progn
      (beginning-of-buffer)
      (let* ((oldpoint (point-max))
	     (absn (or absn ham-fun-default-abs))
	     (anf (or (search-forward absn nil t) (point-max)))
	     (end (or (re-search-forward "\\[.+\\]" nil t) (point-max))))
	(save-restriction
	  (narrow-to-region anf end)
	  (beginning-of-buffer)
	  (if (< anf oldpoint)
	    ;;bekannter Abschnitt
	    (cond ((search-forward ham-fun-autostart-string nil t)
		    (insert "\n" str)
		    (save-buffer)
		    (kill-buffer nil))
		  (t
		    (insert "\n" ham-fun-autostart-string
			  "\n" str "\n" ham-fun-autoend-string)
		    (save-buffer)
		    (kill-buffer nil)))
	    ;;kein bekannter Abschnitt
	    (end-of-buffer)
	    (search-backward "Local Variables" nil t)
	    (beginning-of-line)
	    (insert absn "\n" ham-fun-autostart-string
		    "\n" str "\n" ham-fun-autoend-string "\n" "\n")
	    (save-buffer)
	    (kill-buffer nil)))))))

(defun ham-fun-overwrite-match (string)
  "Loescht aktuelle Zeile. Schreibt uebergebenen String dafuer."
  (kill-region (point-at-bol) (point-at-eol))
  (insert string)
  (save-buffer)
  (kill-buffer nil))

(defun ham-fun-search-string (stringa)
  "Sucht Teile des uebergebenen Strings. Hierzu werden in dem String
alle Ziffern und einige Sonderzeichen geloescht. Aufeinanderfolgende
Leerzeichen werden durch eines ersetzt. Anschliessend werden
Zeichenfolgen eingesetzt, damit eine Regexpsuche moeglich ist.

Der Sinn ist, Subjectstrings, von periodischen Postings, deren
Versionsnummer meist durch ein Datum gekennzeichnet sind zu finden,
dabei wird auf den Text gematched."
  (let* ((stringb (replace-in-string stringa "<[-0-9/\.]+>" ""))
	 (stringc (replace-in-string stringb "([-0-9/\.]+)" ""))
	 (stringd (replace-in-string stringc "[[]" "\\["))
	 (stringe (replace-in-string stringd "[]]" "\\]")))
    (beginning-of-buffer)
    (re-search-forward stringe nil t)))

(defun ham-fun-concat-string (number prefix string suffix &optional exp)
  "Erzeugt Scorestring. Ersetzt `\"' durch `\\\"'."
  (let* ((string (replace-in-string string "\"" "\\\"")))
    (concat number prefix string suffix exp)))

(defun ham-fun-newsscore-subject (&optional cpa score abs subjectstring exp)
  "Schreibt aktuellen Subjectheader in Hamsternewsscorefile.
Default ist ein Score von -10000 und der Eintrag in den Abschnitt [*]
als literaler String. Es wird ein Expire-Feld geschrieben, bei dem der Wert
von `ham-fun-default-expdelta' zu dem aktuellen Datum addiert wird.
Mit dem optionalen numerischen Prefix kann etwas  anderes erzwungen werden.
Ein leeres Prefix (nur C-u) oder ungleich den bekannten Werten, erlaubt einen
freien Eintrag als literaler String ohne expire, ein Prefix von gleich 1 mit expire.
Ein Prefix gleich 2 erlaubt einen freien Eintrag als regexp ohne expire.
Ein Prefix gleich 3 erlaubt einen freien Eintrag als regexp-String, mit frei
whlbarem expire.

Zuerst wird getestet, ob ein Scorefileeintrag vorhanden ist, der sich vom
Suchstring nur in der Ziffernfolge unterscheidet. Ist dies der Fall,
wird der betreffende Eintrag ersetzt. Dabei wird nur der Matchstring
ersetzt, der vorhandene Scoreeintrag bleibt bestehen. Ansonst
wird in die Region zwischeen `ham-fun-autostart-string' und
`ham-fun-autoend-string' der string geschrieben.
Wird diese Region nicht gefunden, werden die Eintraege getaetigt."
  (interactive
   (cond ((eq current-prefix-arg nil)	;default
	  (setq cpa current-prefix-arg))
	 ((eq current-prefix-arg 1)
	  (list
	   (setq cpa current-prefix-arg)
	   (read-string "Score: "  ham-fun-default-score)
	   (read-string "Abschnitt: " (ham-fun-newsgroup-name))
	   (read-string "Literaler Scorestring: " (ham-fun-article-subject))
	   (read-string "Expire (Tagen): " "30")))
	 ((eq current-prefix-arg 2)
	  (list
	   (setq cpa current-prefix-arg)
	   (read-string "Score: "  ham-fun-default-score)
	   (read-string "Abschnitt: " (ham-fun-newsgroup-name))
	   (read-string "Regexp Scorestring: " (ham-fun-article-subject))))
	 ((eq current-prefix-arg 3)
	  (list
	   (setq cpa current-prefix-arg)
	   (read-string "Score: "  ham-fun-default-score)
	   (read-string "Abschnitt: " (ham-fun-newsgroup-name))
	   (read-string "Regexp Scorestring: " (ham-fun-article-subject))
	   (read-string "Expire (Tagen): " "30")))
	 (t				;alles ausser 1,2,3
	  (list
	   (setq cpa current-prefix-arg)
	   (read-string "Score: "  ham-fun-default-score)
	   (read-string "Abschnitt: " (ham-fun-newsgroup-name))
	   (read-string "Literaler Scorestring: " (ham-fun-article-subject))))))
     (let* ((score (or score ham-fun-default-score))
	    (exp (if (or (eq cpa 1) (eq cpa 3) (eq cpa nil))
		     (ham-fun-expire-date exp)
		   nil))
	    (subjectstring (or subjectstring (ham-fun-article-subject)))
	    (prefix
	     (if (or (eq cpa 2) (eq cpa 3))
		 ham-fun-news-subject-regexp-prefix
	       ham-fun-news-subject-lit-prefix))
	    (suffix
	     (if (or (eq cpa 2) (eq cpa 3))
		 ham-fun-score-regexp-suffix
	       ham-fun-score-lit-suffix))
	  (scorestring (ham-fun-concat-string
			score prefix subjectstring suffix exp)))
     (ham-fun-scorefile-open
      ham-fun-root-directory ham-fun-news-scorefile)
     (if (ham-fun-search-string subjectstring)
	 (ham-fun-overwrite-match scorestring)
       (ham-fun-scorestring-insert scorestring nil abs))))

(defun ham-fun-newsscore-from (&optional cpa score abs fromstring exp)
  "Schreibt aktuellen From-Headers in das Hamsternewsscorefile.
Default ist ein Score von -10000 und der Eintrag in den Abschnitt [*]
als literaler String. Es wird ein Expire-Feld geschrieben, bei dem der Wert
von `ham-fun-default-expdelta' zu dem aktuellen Datum addiert wird.
Mit dem optionalen numerischen Prefix kann etwas  anderes erzwungen werden.
Ein leeres Prefix (nur C-u) oder ungleich den bekannten Werten, erlaubt einen
freien Eintrag als literaler String ohne expire, ein Prefix von gleich 1 mit expire.
Ein Prefix gleich 2 erlaubt einen freien Eintrag als regexp ohne expire.
Ein Prefix gleich 3 erlaubt einen freien Eintrag als regexp-String, mit frei
whlbarem expire.

Wird ein Eintrag gefunden, der dem aktuellen Match-String aehnelt, wird
unter diesen Eintrag ein neuer Eintrag gesetzt, sonst in die Region, die
von `ham-fun-autostart-string' und `ham-fun-autoend-string' begrenzt ist.
Wird diese Region nicht gefunden, werden die Eintraege getaetigt."
  (interactive
   (cond ((eq current-prefix-arg nil)	;default
	  (setq cpa current-prefix-arg))
	 ((eq current-prefix-arg 1)
	  (list
	   (setq cpa current-prefix-arg)
	   (read-string "Score: "  ham-fun-default-score)
	   (read-string "Abschnitt: " (ham-fun-newsgroup-name))
	   (read-string "Literaler Scorestring: " (ham-fun-article-from))
	   (read-string "Expire (Tagen): " "30")))
	 ((eq current-prefix-arg 2)
	  (list
	   (setq cpa current-prefix-arg)
	   (read-string "Score: "  ham-fun-default-score)
	   (read-string "Abschnitt: " (ham-fun-newsgroup-name))
	   (read-string "Regexp Scorestring: " (ham-fun-article-from))))
	 ((eq current-prefix-arg 3)
	  (list
	   (setq cpa current-prefix-arg)
	   (read-string "Score: "  ham-fun-default-score)
	   (read-string "Abschnitt: " (ham-fun-newsgroup-name))
	   (read-string "Regexp Scorestring: " (ham-fun-article-from))
	   (read-string "Expire (Tagen): " "30")))
	 (t				;alles ausser 1,2,3
	  (list
	   (setq cpa current-prefix-arg)
	   (read-string "Score: "  ham-fun-default-score)
	   (read-string "Abschnitt: " (ham-fun-newsgroup-name))
	   (read-string "Literaler Scorestring: " (ham-fun-article-from))))))
  (let* ((score (or score ham-fun-default-score))
	 (exp (if (or (eq cpa 1) (eq cpa 3) (eq cpa nil))
		  (ham-fun-expire-date exp)
		nil))
	 (fromstring (or fromstring (ham-fun-article-from)))
	 (prefix
	  (if (or (eq cpa 2) (eq cpa 3))
	      ham-fun-news-from-regexp-prefix
	    ham-fun-news-from-lit-prefix))
	 (suffix
	  (if (or (eq cpa 2) (eq cpa 3))
	      ham-fun-score-regexp-suffix
	    ham-fun-score-lit-suffix))
	 (scorestring (ham-fun-concat-string
		       score prefix fromstring suffix exp)))
    (ham-fun-scorefile-open
     ham-fun-root-directory ham-fun-news-scorefile)
	(if (ham-fun-search-string fromstring)
	    (ham-fun-scorestring-insert scorestring t)
	  (ham-fun-scorestring-insert scorestring nil abs))))

(defun ham-fun-kill-mid ()
  "Schreibt aktuelle M-ID in eine Datei.
Diese Datei wird von einem Hamsterskript ausgewertet."
  (interactive)
  (if (gnus-news-group-p gnus-newsgroup-name)
    (let ((mid (ham-fun-article-mid)))
      (save-excursion
	(ham-fun-scorefile-open
	 ham-fun-root-directory ham-fun-mid-killfile)
	(insert mid "\n")
	(save-buffer)
	(kill-buffer nil))
      (gnus-summary-mark-as-read-forward 1))
    (gnus-summary-delete-article)
    (newline)))

(defun ham-fun-scorefile-edit ()
  "Oeffnet Scorefile zum bearbeiten. Stammt die aktuelle Gruppe von einem
Newsserver wird `ham-fun-news-scorefile' sonst `ham-fun-mail-filterfile'
geoeffnet."
  (interactive)
  (if (gnus-news-group-p gnus-newsgroup-name)
      (find-file (concat ham-fun-root-directory ham-fun-news-scorefile))
    (find-file (concat ham-fun-root-directory ham-fun-mail-filterfile))))

(provide 'hamster-functions)

